//
//  MergeVideoControllerViewController.m
//  VideoEditDemo
//
//  Created by Om's MINI on 11/23/12.
//  Copyright (c) 2012 Mobisoft's mini. All rights reserved.
//

#import "CustomVideoOverlapper.h"
#import "CustomMoviePlayer.h"
#import "MBProgressHUD.h"

@interface CustomVideoOverlapper () {
    
    NSURL *url;
    AVURLAsset *firstAsset;
    AVURLAsset *secondAsset;
    AVMutableVideoComposition *mainComposition;
    AVMutableComposition *mixComposition;
    AVPlayerItem *newPlayerItem;
    
}
@end

@implementation CustomVideoOverlapper

@synthesize controller = _controller;

#pragma mark - Initialisation

- (id)initWithController:(UIViewController *)controller {
    
    self = [super init];
    if (self) {
        
        self.controller = controller;
    }
    return self;
}



#pragma mark - Private Methods

- (void)videoOverlappingMethodWithVideoOneURL:(NSURL *)videoOneURL andVideoTwoURL:(NSURL *)videoTwoURL {
    
    
      //addedd ProgressHud
    [MBProgressHUD showHUDAddedTo:self.controller.view animated:YES];

    if (videoOneURL!= nil && videoTwoURL!=nil) {
    
        
        //First Video
        //firstAsset = [AVURLAsset URLAssetWithURL:[NSURL fileURLWithPath:[[NSBundle mainBundle] pathForResource:@"sample1" ofType:@"m4v"]] options:nil];
        NSLog(@"First Asset = %@",firstAsset);
   
        firstAsset = [AVAsset assetWithURL:videoOneURL];
        
        //second Video
        //secondAsset = [AVURLAsset URLAssetWithURL:[NSURL fileURLWithPath:[[NSBundle mainBundle] pathForResource:@"sample2" ofType:@"m4v"]] options:nil];
         NSLog(@"second Asset = %@",secondAsset);
         secondAsset = [AVAsset assetWithURL:videoTwoURL];
    }
    
    if (firstAsset != nil && secondAsset != nil) {
                
        //Create AVMutableComposition object
        mixComposition = [[AVMutableComposition alloc] init];
        
        // create first track
        AVMutableCompositionTrack *firstTrack =
               [mixComposition addMutableTrackWithMediaType:AVMediaTypeVideo
                                           preferredTrackID:kCMPersistentTrackID_Invalid];
        
        [firstTrack insertTimeRange:CMTimeRangeMake(kCMTimeZero, firstAsset.duration)
                            ofTrack:[[firstAsset tracksWithMediaType:AVMediaTypeVideo] objectAtIndex:0]
                             atTime:kCMTimeZero
                              error:nil];
        
        //create second track

        AVMutableCompositionTrack *secondTrack = [mixComposition addMutableTrackWithMediaType:AVMediaTypeVideo preferredTrackID:kCMPersistentTrackID_Invalid];
        
        [secondTrack insertTimeRange:CMTimeRangeMake(kCMTimeZero, secondAsset.duration)
                             ofTrack:[[secondAsset tracksWithMediaType:AVMediaTypeVideo] objectAtIndex:0]
                              atTime:kCMTimeZero
                               error:nil];
        
        
        //Main Instruction Layer
        //crete Object of main Instruction layer and set timerange to it
        
        AVMutableVideoCompositionInstruction *mainInstruction = [AVMutableVideoCompositionInstruction videoCompositionInstruction];
        
        //Duraton For Final Video should be max Duration of both video
        CMTime finalDuration;
        CMTime result;
        
        NSLog(@"values =%f and %f",CMTimeGetSeconds(firstAsset.duration),CMTimeGetSeconds(secondAsset.duration));
        
        if (CMTimeGetSeconds(firstAsset.duration) == CMTimeGetSeconds(secondAsset.duration)) {
            
            finalDuration = firstAsset.duration;
            
        }else if (CMTimeGetSeconds(firstAsset.duration) > CMTimeGetSeconds(secondAsset.duration)) {
            
            finalDuration = firstAsset.duration;
            result = CMTimeSubtract(firstAsset.duration, secondAsset.duration);
            
        }else {
            
            finalDuration = secondAsset.duration;
            result = CMTimeSubtract(secondAsset.duration, firstAsset.duration);
            
        }

        //CMTime duration = MAX(secondAsset.duration , firstAsset.duration);
        mainInstruction.timeRange = CMTimeRangeMake(kCMTimeZero,finalDuration);
        
        //InstructionLayer for first Track
        
        AVMutableVideoCompositionLayerInstruction *firstlayerInstruction = [AVMutableVideoCompositionLayerInstruction videoCompositionLayerInstructionWithAssetTrack:firstTrack];
        
        CGAffineTransform scale = CGAffineTransformMakeScale(0.5f, 0.5f);
        CGAffineTransform move = CGAffineTransformMakeTranslation(320, 0);
        
        [firstlayerInstruction setTransform:CGAffineTransformConcat(scale, move) atTime:kCMTimeZero];
        
        //InstructionLayer for second Track
        
        AVMutableVideoCompositionLayerInstruction *secondlayerInstruction = [AVMutableVideoCompositionLayerInstruction videoCompositionLayerInstructionWithAssetTrack:secondTrack];
        
        CGAffineTransform secondScale = CGAffineTransformMakeScale(1.2f,1.5f);
        
        CGAffineTransform secondMove = CGAffineTransformMakeTranslation(0,0);
        
        //[secondlayerInstruction setOpacity:0.5 atTime:kCMTimeZero];
        
        [secondlayerInstruction setOpacityRampFromStartOpacity:0.0 toEndOpacity:1.0 timeRange:CMTimeRangeMake(CMTimeMakeWithSeconds(2.9,60), CMTimeMakeWithSeconds(8.0, 60))];
        
        [secondlayerInstruction setTransform:CGAffineTransformConcat(secondScale,secondMove) atTime:kCMTimeZero];
                
        //Now adding FirstInstructionLayer and SecondInstructionLayer to mainInstruction
        
        mainInstruction.layerInstructions = [NSArray arrayWithObjects:firstlayerInstruction,secondlayerInstruction, nil];
        
        // attach Main Instrcution To VideoCopositionn
        //we can attch multipe Instrction to it
        
        mainComposition = [AVMutableVideoComposition videoComposition];
        mainComposition.instructions = [NSArray arrayWithObjects:mainInstruction,nil];
        mainComposition.frameDuration = CMTimeMake(1, 30);
        mainComposition.renderSize = CGSizeMake(640, 480);
        
        //  Get path
        
        NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
        
        NSString *documentsDirectory = [paths objectAtIndex:0];
        
        NSString *myPathDocs =  [documentsDirectory stringByAppendingPathComponent:
                                 [NSString stringWithFormat:@"mergeVideo.mov"]];
        
        NSFileManager *fileManager = [NSFileManager defaultManager];
        [fileManager removeItemAtPath:myPathDocs error:NULL];
        
        url = [NSURL fileURLWithPath:myPathDocs];
        
        NSLog(@"URL:-  %@", [url description]);
        
        //create Exporter
        
        AVAssetExportSession *exporter = [[AVAssetExportSession alloc] initWithAsset:mixComposition presetName:AVAssetExportPresetHighestQuality];
        
        exporter.outputURL = url;
        
        exporter.outputFileType = AVFileTypeQuickTimeMovie;
        
        exporter.shouldOptimizeForNetworkUse = YES;
        
        exporter.videoComposition = mainComposition;
        
        [exporter exportAsynchronouslyWithCompletionHandler:^{
            
            dispatch_async(dispatch_get_main_queue(), ^{
                
                [self exportDidFinish:exporter];
                
            });
        }];
        
    }else {
        
        [MBProgressHUD hideHUDForView:self.controller.view animated:YES];
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error!" message:@"Video Not Selected.PLease select videos to merege"
                                                       delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
    }

}

-(void)exportDidFinish:(AVAssetExportSession*)session {
    
    NSLog(@"exportDidFinish");
    
    //Remove ProgressHud
    [MBProgressHUD hideHUDForView:self.controller.view animated:YES];
    NSLog(@"session = %d",(int)session.status);
    if (session.status == AVAssetExportSessionStatusCompleted) {
        
        NSURL *outputURL = session.outputURL;
        
        ALAssetsLibrary *library = [[ALAssetsLibrary alloc] init];
        
        if ([library videoAtPathIsCompatibleWithSavedPhotosAlbum:outputURL])  {
            
            [library writeVideoAtPathToSavedPhotosAlbum:outputURL completionBlock:^(NSURL *assetURL, NSError *error) {
                
                dispatch_async(dispatch_get_main_queue(), ^{
                    
                    if (error) {
                        
                        NSLog(@"writeVideoToAssestsLibrary failed: %@", error);
                        
                        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error"
                                                                        message:@"Video Saving Failed"
                                                                       delegate:nil
                                                              cancelButtonTitle:@"OK"
                                                              otherButtonTitles:nil];
                        [alert show];
                        
                    }else {
                        
                        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Video Saved"
                                                                        message:@"Saved To Photo Album"
                                                                       delegate:self
                                                              cancelButtonTitle:@"OK"
                                                              otherButtonTitles:nil];
                        alert.tag = 1000;
                        [alert show];
                        
                        NSLog(@"Writing3");
                        
                    }

                    
                });
            }];
            
        }

    }else {
        
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error!"
                                                        message:@"Video asset export failed"
                                                       delegate:self
                                              cancelButtonTitle:@"OK"
                                              otherButtonTitles:nil];
        [alert show];
    }
    
}

#pragma mark - UIAlertView Delegate Method

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex  {
    
    if (alertView.tag == 1000) {
        
        if (buttonIndex == 0) {
            
         UIAlertView *playAlert = [[UIAlertView alloc] initWithTitle:@"Play Video"
                                                             message:@"Dou you want play it immdiately?"
                                                            delegate:self
                                                   cancelButtonTitle:@"Yes"
                                                   otherButtonTitles:@"later", nil];
            playAlert.tag = 2000;
            [playAlert show];
            
        }else {
            
        }

    }else if(alertView.tag == 2000){
        
        if (buttonIndex == 0) {
            
            CustomMoviePlayer *moviePlayer = [[CustomMoviePlayer alloc] initWithController:self.controller andURLForMovie:url];
            
            [moviePlayer playVideo];
          
            }
            
        }
    
}

@end
